/* dc_gui2 - a GTK+2 GUI for DCTC
 * Copyright (C) 2002 Eric Prevoteau
 *
 * bittorrent_clist.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
$Id: bittorrent_clist.c,v 1.2 2003/08/16 15:47:30 ericprev Exp $
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <dirent.h>
#include <gnome.h>

#include "gtkcellrenderertextx.h"
#include "misc_gtk.h"
#include "bittorrent_clist.h"
#include "main.h"
#include "gui_define.h"
#include "macro.h"
#include "bt2dc_gui2_io.h"

typedef struct
{
	gboolean in_dir;		/* TRUE if the entry is visible in the directory */
	gboolean in_list;		/* TRUE if the entry is in the clist */
	gboolean is_new;		/* TRUE if the entry was just created */
	int bt_pid;
	time_t last_update;
	BT_DL_STRUCT *map;	/* mapped file */
} BT_ENTRY;

static GArray *available_bt_entry=NULL;

/***************************************************************************************/
/* set the in_dir and is_new flags of all entries of available_bt_entry array to FALSE */
/***************************************************************************************/
static void clear_dir_and_new_flags(void)
{
	int i;
	for(i=0;i<available_bt_entry->len;i++)
	{
		g_array_index(available_bt_entry,BT_ENTRY,i).in_dir=FALSE;
		g_array_index(available_bt_entry,BT_ENTRY,i).is_new=FALSE;
	}
}

/************************************************************************************/
/* get the array index of an entry of the available_bt_entry array from the process */
/************************************************************************************/
/* output: array index or -1 */
/*****************************/
static int get_idx_by_pid(int pid)
{
	int i;

	for(i=0;i<available_bt_entry->len;i++)
	{
		if(g_array_index(available_bt_entry,BT_ENTRY,i).bt_pid==pid)
			return i;
	}
	return -1;
}

/************************************/
/* open and map the given new entry */
/************************************/
static void add_new_entry(const char *filepath, int proc_pid)
{
	int fd;
	BT_ENTRY new_bt;

	fd=open(filepath,O_RDWR);
	if(fd==-1)
	{
		fprintf(stderr,"Fail to open file '%s': %s\n",filepath,strerror(errno));
		return;
	}

	new_bt.in_dir=TRUE;
	new_bt.in_list=FALSE;
	new_bt.is_new=TRUE;
	new_bt.bt_pid=proc_pid;
	new_bt.last_update=0;
	new_bt.map=mmap(NULL,sizeof(BT_DL_STRUCT),PROT_READ|PROT_WRITE,MAP_SHARED, fd,0);
	if(new_bt.map==MAP_FAILED)
	{
		fprintf(stderr,"Fail to map file '%s': %s\n",filepath,strerror(errno));
		close(fd);
		return;
	}

	close(fd);
	g_array_append_val(available_bt_entry,new_bt);
}

/**************************************************************************/
/* remove all entries with in_dir=FALSE from the clist and from the array */
/**************************************************************************/
static void remove_dead_entries(void)
{
	GtkTreeView *gtv;
	GtkTreeModel *gtm;
	GtkListStore *gls;
	GtkTreeIter iter;
	GtkWidget *w;
	int i;

	w=get_widget_by_widget_name(main_window,"bittorrent_treeview");
	gls=GTK_LIST_STORE(gtm=gtk_tree_view_get_model(gtv=GTK_TREE_VIEW(w)));

	i=gtk_tree_model_iter_n_children(gtm,NULL)-1;
	while(i>=0)
	{
		if(gtk_tree_model_iter_nth_child(gtm,&iter,NULL,i)==TRUE)
		{
			gint proc_pid;
			int idx;
			
			gtk_tree_model_get(gtm,&iter,BTC_BITTORRENT_PID,&proc_pid,-1);
			if((idx=get_idx_by_pid(proc_pid))==-1)
			{
				/* entry no more exists */
				gtk_list_store_remove(gls,&iter);		/* delete it from the list */
																	/* and delete its entry */

				munmap(g_array_index(available_bt_entry,BT_ENTRY,idx).map,sizeof(BT_DL_STRUCT));
				g_array_remove_index_fast(available_bt_entry,idx);
			}

			i--;
		}
	}
}

/**********************************************************************************************************/
/* update the content of the bittorrent clist with the content of the directory $HOME/.dc_gui2/bt2dc_gui2 */
/**********************************************************************************************************/
void reload_bittorrent_clist(void)
{
	DIR *dir;
	struct dirent *obj;
	int base_len;
	GString *fpath;

	dir=opendir(bt2dc_gui2_directory->str);
	if(dir==NULL)
		return;

	if(available_bt_entry==NULL)
	{
		available_bt_entry=g_array_new(FALSE,FALSE,sizeof(BT_ENTRY));
	}

	clear_dir_and_new_flags();

	fpath=g_string_new(bt2dc_gui2_directory->str);
	base_len=fpath->len;

	while((obj=readdir(dir))!=NULL)
	{
		int proc_pid;
		int idx;

		if(obj->d_name[0]=='.')
			continue;

		if(strncmp(obj->d_name,"bt_xfer.",8))
			continue;

		proc_pid=atoi(obj->d_name+8);
		idx=get_idx_by_pid(proc_pid);
		if(idx!=-1)
		{
			g_array_index(available_bt_entry,BT_ENTRY,idx).in_dir=FALSE;
		}
		else
		{
			g_string_append(fpath,obj->d_name);
			add_new_entry(fpath->str,proc_pid);
			g_string_truncate(fpath,base_len);
		}
	}
	g_string_free(fpath,TRUE);
	closedir(dir);

	/* now, we have 3 tasks to do: */
	/* 1) for each entry of the array with in_dir=FALSE, remove them from the clist and from the array */
	/* 2) for each entry of the array with in_list=TRUE, check if they are still running and update their entry in the clist (and the time in the array) */
	/* 3) for each entry of the array with is_new=TRUE, create them in the clist */
	remove_dead_entries();
}

/**************************************************/
/* build modele and view for the queue user clist */
/**************************************************/
void bmav4_bittorrent_clist(void)
{
	GtkListStore *model;
	GtkWidget *view;
	GtkCellRenderer *rend;
	GtkTreeSelection *slc;

	model=gtk_list_store_new(NB_BTC_COL,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_ULONG,
													G_TYPE_STRING,G_TYPE_STRING,G_TYPE_FLOAT,
													G_TYPE_FLOAT,G_TYPE_ULONG,G_TYPE_STRING, G_TYPE_INT);

	view=get_widget_by_widget_name(main_window,"bittorrent_treeview");

	gtk_tree_view_set_model(GTK_TREE_VIEW(view),GTK_TREE_MODEL(model));
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view),TRUE);

	slc=gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
	gtk_tree_selection_set_mode(slc,GTK_SELECTION_MULTIPLE);
	
	/* the first column of the view display the first column of the model, and so on. The 4th entry of the model is not displayed */
	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),BTC_FILENAME_COL,_("Filename"),rend,"text",BTC_FILENAME_COL,"background",BTC_BACKGROUND_COL,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),BTC_SIZE_COL,_("Size"),rend,"text",BTC_SIZE_COL,"background",BTC_BACKGROUND_COL,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),BTC_STATUS_COL,_("Status"),rend,"text",BTC_STATUS_COL,"background",BTC_BACKGROUND_COL,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),BTC_DL_SPEED_COL,_("DL spd"),rend,"text",BTC_DL_SPEED_COL,"background",BTC_BACKGROUND_COL,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),BTC_UL_SPEED_COL,_("UL spd"),rend,"text",BTC_UL_SPEED_COL,"background",BTC_BACKGROUND_COL,NULL);

/* The view now holds a reference.  We can get rid of our own
	 * reference */
	g_object_unref (G_OBJECT (model));
}

